#!/usr/bin/perl
$Usage = "$0 [-h] [-verbose] [Aria/]city ...\n";

$ZoneinfoDir = "/usr/share/zoneinfo";
@WeekdayName = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat");
@MonthName = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
$Verbose = 0;

#--- interpret the command-line arguments
while ($ARGV[0]) {
    # print STDERR "$ARGV[0]\n";
    if ($ARGV[0] =~ /^-D.*/) {
	$Debug = 1;
    } elsif ($ARGV[0] =~ /^-v.*/) {
	$Verbose = 1;
    } elsif ($ARGV[0] =~ /^-.*/) {
	print STDERR "$Usage";
	exit 0;
    } else {
	last;
    }
    shift(@ARGV);
}

#--- checking to set the necessary variables.
#--- DebugPrint function
#$Debug = 1;
sub Dprint { if (defined($Debug)) { printf STDERR "DBG @_"; } return 0; }
Dprint "ARGS($#ARGV): @ARGV\n";

#--- Current time of Greenwich.
$gm_time = time() - 9 * 60 * 60;

#--- Check zoneinfo dir
if (!-d $ZoneinfoDir) {
    my @zdirs = `locate /zoneinfo/Asia/Tokyo.ics`;
    #-- Use the first one without reasons.
    if ($#zdirs < 0 || $zdirs[0] !~ m|^(.*)/[^/]+/[^/]+$/|) {
	printf STDERR "$0 zoneinfo directory is not found.";
	exit(-1);
    }
    $ZoneinfoDir = $1;
    Dprint "ZoneinfoDir ($ZoneinfoDir)\n";
}

#--- Create file list
foreach my $arg (@ARGV) {
    my @matchs_complete = ();
    my @matchs_partial = ();
    my $sout = "";
    
    if ($arg =~ m|/|) {
	#-- Asia/Katmandu
	if (-f "$ZoneinfoDir/$arg.ics") {
	    push(@matchs_complete, "$ZoneinfoDir/$arg.ics");
	}
	@matchs_partial  = glob("$ZoneinfoDir/${arg}?*.ics");
    } else {
	#-- Katmandu
	@matchs_complete = glob("$ZoneinfoDir/*/$arg.ics");
	@matchs_partial  = glob("$ZoneinfoDir/*/${arg}?*.ics");
    }
    Dprint "Complete: (@matchs_complete)\n";
    Dprint "Partial : (@matchs_partial)\n";
    
    foreach my $zfile (@matchs_complete) {
	$sout = GetLocalTimeString($zfile);
	print "$sout\n";
    }
    foreach my $zfile (@matchs_partial) {
	$sout = GetLocalTimeString($zfile);
	print "$sout\n";
    }
}

exit 0;

#---
sub GetLocalTimeString {
    my ($zf) = @_;
    my $srtn;
    open(IN, "<$zf") or return "";
    my @czf = <IN>;
    close(IN);
    my ($tzsign, $tzhour, $tzmin, $tzname) = ("", "", "");
    foreach (@czf) {
	if (m|TZOFFSETTO:([+-])([0-9]{2})([0-9]{2})|i) {
	    ($tzsign, $tzhour, $tzmin) = ($1, $2, $3);
	} elsif (m|X-LIC-LOCATION:(.+)|i) {
	    $tzlocation = $1;
	} elsif (m|TZNAME:([a-zA-Z]{3})|i) {
	    $tzname = $1;
	}
    }
    Dprint "($tzsign, $tzhour, $tzmin, $tzname)\n";
    if (length($tzhour) <= 0 || length($tzmin) <= 0 || length($tzsign) <= 0) {
	return "";
    }
    my $lc_time;
    if ($tzsign eq "-") {
	$lc_time = $gm_time - ($tzhour * 60 + $tzmin) * 60;
    } else {
	$lc_time = $gm_time + ($tzhour * 60 + $tzmin) * 60;
    }
    Dprint "($gm_time, $lc_time)\n";
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($lc_time);
    if ($isdst) {
	#-- The aria supports summer time.
	($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($lc_time + 60 * 60);
	$tzlocation .= " (ST)";
    }
    #$srtn = "$WeekdayName[$wday] " . $MonthName[$mon] . " " . Fmt($mday, 2) . " " . Fmt($hour, 2) . ":" . Fmt($min, 2) . ":" . Fmt($sec, 2) . " " . ($year + 1900) . " $tzlocation";
    $srtn = Fmt($mon + 1, 2) . "/" . Fmt($mday, 2) . " $WeekdayName[$wday] " . Fmt($hour, 2) . ":" . Fmt($min, 2) . ":" . Fmt($sec, 2) . " " . ($year + 1900) . " $tzname  " . "$tzlocation";

    return $srtn;
}

#--- 
sub Fmt {
    my ($nbr, $kta) = @_;
    return sprintf("%0${kta}d", $nbr);
}

	
